//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
//
//	delib_io_tcp.cpp
//	project: delib
//
//
//	(c) DEDITEC GmbH, 2009-2017
//	web: http://www.deditec.de/
//	mail: vertrieb@deditec.de
//
//
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

// ------------------------------------
// DEBUG OUTPUT 
//#define DEBUG_OUTPUT		1				// Ausgabe aktivieren (nur fr diese CPP-Datei)
// ------------------------------------

// ------------------------------------
// ------------------------------------
// History
// ------------------------------------
// ------------------------------------
//
// DT	13.11.2013	Start DEDITEC-History



// ------------------------------------
// ------------------------------------
// Functions
// ------------------------------------
// ------------------------------------


// ------------------------------------
// tcp_tx_open
// tcp_tx_send_rec_data
// tcp_tx_close
// tcp_rx_close_session
// tcp_tx_send_rec_data_big

// ------------------------------------
// ------------------------------------
// Includes
// ------------------------------------
// ------------------------------------

#include <stdlib.h>
#include <stdio.h>

#include "../library/communication/dt_communication_paramams.h"

#include "../library/embedded/dt_var_defines.h"
#include "../library/delib/delib_error_codes.h"

// alle sachen frs DAPI_HANDLE
#include "../library/vc/deditec_tcp_io.h"		// TCP_IO
#include "../../usb-ftdi/ftd2xx.h"				// FT_HANDLE
#include "../library/delib/delib.h"				// ULONG, FUNCTION_PRE, ..
#include "../library/delib/delib_intern.h"		// DAPI_HANDLE

#include "../library/vc/deditec_string_utils.h"		// strcpy_n
#include "../library/vc/deditec_buff_utils.h"		// buffcpy_n
#include "deditec_encryption_utils.h"

// ------------------------------------
// ------------------------------------
// Defines
// ------------------------------------
// ------------------------------------

#define TCP_RO_TX_ENCRYPTION_START		4		// id(2) + ro-cmd(1) + job_id(1)
#define TCP_RO_RX_ENCRYPTION_START		5		// id(2) + answer(1) + ok(1) + job_id(1)

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_encryption_get_info(	DAPI_HANDLE *DapiHandle,
										char *encryption_pw, unsigned long encryption_pw_size,
										unsigned char *packet_id, bool *encryption_needed)
{
	bool encryption_flag = FALSE;
	unsigned char pkt_id;
	
	switch (DapiHandle->encryption_type)
	{
		// ------------------------------------
		case DAPI_OPEN_MODULE_ENCRYPTION_TYPE_NONE:		
			pkt_id = (unsigned char) DEDITEC_TCP_PACKET_ID_1_NORMAL;
			break;
		// ------------------------------------
		case DAPI_OPEN_MODULE_ENCRYPTION_TYPE_NORMAL:
			if (encryption_pw != NULL)
			{
				strcpy_n((char*) encryption_pw, encryption_pw_size, (char*) DapiHandle->encryption_password, strlen_n((char*) DapiHandle->encryption_password, sizeof(DapiHandle->encryption_password)));
			}
			pkt_id = (unsigned char) DEDITEC_TCP_PACKET_ID_1_ENCRYPT_NORMAL;
			encryption_flag = TRUE;
			break;		
		// ------------------------------------
		case DAPI_OPEN_MODULE_ENCRYPTION_TYPE_ADMIN:
			if (encryption_pw != NULL)
			{
				strcpy_n((char*) encryption_pw, encryption_pw_size, (char*) DapiHandle->encryption_password, strlen_n((char*) DapiHandle->encryption_password, sizeof(DapiHandle->encryption_password)));
			}
			pkt_id = (unsigned char) DEDITEC_TCP_PACKET_ID_1_ENCRYPT_ADMIN;
			encryption_flag = TRUE;
			break;
		// ------------------------------------
		case DAPI_OPEN_MODULE_ENCRYPTION_TYPE_ADMIN_TEMP:
			if (encryption_pw != NULL)
			{
				strcpy_n((char*) encryption_pw, encryption_pw_size, DEDITEC_ENCRYPTION_TEMPORARY_ADMIN_PW, sizeof(DEDITEC_ENCRYPTION_TEMPORARY_ADMIN_PW));
			}
			pkt_id = (unsigned char) DEDITEC_TCP_PACKET_ID_1_ENCRYPT_ADMIN_TEMP;
			encryption_flag = TRUE;
			break;
		// ------------------------------------
		default:
			return DAPI_ERR_GEN_UNKNOWN_ENCRYPTION_TYPE;
		// ------------------------------------
	}
	
	if (packet_id != NULL)
	{
		*packet_id = pkt_id;
	}
	
	if (encryption_needed != NULL)
	{
		*encryption_needed = encryption_flag;
	}
	
	return DAPI_ERR_NONE;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_encrypt_tcp_pre(DAPI_HANDLE *DapiHandle, unsigned char *tcp_msg, unsigned long *tcp_msg_len, unsigned long *tcp_msg_pos)
{
	bool encryption_flag;
	unsigned long ret;
	
	if ((ret = dt_encryption_get_info(DapiHandle, NULL, 0, NULL, &encryption_flag)) != DAPI_ERR_NONE)
	{
		return ret;
	}
	
	if (encryption_flag == TRUE)
	{
		if (buffcpy_n((tcp_msg + TCP_RO_TX_ENCRYPTION_START), TCP_RO_TX_ENCRYPTION_START, tcp_msg, TCP_RO_TX_ENCRYPTION_START) != RETURN_OK)
		{
			return DAPI_ERR_GEN_BUFFER_TOO_SMALL_ERROR;
		}
		
		*tcp_msg_len += TCP_RO_TX_ENCRYPTION_START;
		*tcp_msg_pos += TCP_RO_TX_ENCRYPTION_START;
	}
	
	return DAPI_ERR_NONE;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_encrypt_tcp_msg(DAPI_HANDLE *DapiHandle, unsigned char *tcp_msg, unsigned long *tcp_msg_len)
{
	unsigned long encryption_length;
	unsigned long remainder;
	char encryption_pw[32];
	bool encryption_flag;
	unsigned long ret;
	
	if ((ret = dt_encryption_get_info(DapiHandle, encryption_pw, sizeof(encryption_pw), NULL, &encryption_flag)) != DAPI_ERR_NONE)
	{
		return ret;
	}
	
	if (encryption_flag == TRUE)
	{
		// first 4bytes (0=id0, 1=id1, 2=cmd, 3=jobid) won't be encrypted -> (*tcp_msg_len-4) ... decryption starts at tx_buffer[4]
		encryption_length = *tcp_msg_len - TCP_RO_TX_ENCRYPTION_START;

		// length of encryption buffer must be a multiple of 8!
		if ((remainder = (encryption_length % 8)) != 0)
		{
			// not a multiple of 8 -> fix encryption_length and set new *tcp_msg_len
			encryption_length += (8 - remainder);
			*tcp_msg_len = encryption_length + TCP_RO_TX_ENCRYPTION_START;
		}
		
		if (dt_encrypt_buffer((char*) encryption_pw, strlen_n((char*) encryption_pw, sizeof(encryption_pw)), (tcp_msg + TCP_RO_TX_ENCRYPTION_START), encryption_length) != RETURN_OK)
		{
			return DAPI_ERR_GEN_ENCRYPTION_ERROR;
		}
	}
	
	return DAPI_ERR_NONE;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_decrypt_tcp_pre(DAPI_HANDLE *DapiHandle, unsigned long *tcp_msg_size, unsigned long *tcp_msg_pos)
{
	bool encryption_flag;
	unsigned long remainder;
	unsigned long ret;
	
	if ((ret = dt_encryption_get_info(DapiHandle, NULL, 0, NULL, &encryption_flag)) != DAPI_ERR_NONE)
	{
		return ret;
	}
	
	if (encryption_flag == TRUE)
	{
		*tcp_msg_size += TCP_RO_RX_ENCRYPTION_START;
		*tcp_msg_pos += TCP_RO_RX_ENCRYPTION_START;
		
		// first 5bytes (0=id0, 1=id1, 2=cmd, 3=ok, 4=jobid) won't be decrypted -> (tcp_msg_size-5) ... decryption starts at tcp_msg_size[5]
		if ((remainder = (*tcp_msg_size - TCP_RO_RX_ENCRYPTION_START) % 8) != 0)
		{
			// not a multiple of 8 -> set new tcp_msg_size
			*tcp_msg_size += (8 - remainder);
		}
	}
	
	return DAPI_ERR_NONE;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

unsigned long dt_decrypt_tcp_msg(DAPI_HANDLE *DapiHandle, unsigned char *tcp_msg, unsigned long tcp_msg_len)
{
	bool encryption_flag;
	unsigned long remainder;
	unsigned long ret;
	char encryption_pw[32];
	unsigned long encryption_length;
	
	if ((ret = dt_encryption_get_info(DapiHandle, encryption_pw, sizeof(encryption_pw), NULL, &encryption_flag)) != DAPI_ERR_NONE)
	{
		return ret;
	}
	
	if (encryption_flag == TRUE)
	{
		// first 5bytes (0=id0, 1=id1, 2=cmd, 3=ok, 4=jobid) won't be decrypted -> (tcp_msg_len-5) ... decryption starts at tcp_msg[5]
		encryption_length = tcp_msg_len - TCP_RO_RX_ENCRYPTION_START;
		
		if ((remainder = (encryption_length % 8)) != 0)
		{
			// not a multiple of 8 -> set new tcp_msg_size_size
			encryption_length += (8 - remainder);
		}
		
		if (dt_decrypt_buffer((char*) encryption_pw, strlen_n((char*) encryption_pw, sizeof(encryption_pw)), (tcp_msg + TCP_RO_RX_ENCRYPTION_START), encryption_length) != RETURN_OK)
		{
			return DAPI_ERR_GEN_ENCRYPTION_ERROR;
		}
		
		// check ob passwort stimmt -> hier mssen tcp_msg_size [0..4] = tcp_msg_size[5..9] sein (=4bytes: 0=id0, 1=id1, 2=cmd, 3=errorcode, 4=jobid)
		if (buffcmp_n(tcp_msg, TCP_RO_RX_ENCRYPTION_START, (tcp_msg+TCP_RO_RX_ENCRYPTION_START), TCP_RO_RX_ENCRYPTION_START) != RETURN_OK)
		{
			return DAPI_ERR_DEV_ENCRYPTED_HEADER_NOT_OK;
		}
	}
	
	return DAPI_ERR_NONE;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

